@@ -8,7 +8,11 @@ g3a_naturalmortality_exp <- function (
         list(param_f = param_f, action_step_size_f = action_step_size_f))
 }
 
-g3a_naturalmortality <- function (stock, mortality_f, run_f = TRUE, run_at = 4) {
+g3a_naturalmortality <- function (
+        stock,
+        mortality_f = g3a_naturalmortality_exp(),
+        run_f = TRUE,
+        run_at = 4) {
     # See Stock::reducePop, NaturalMortality::Reset
     out <- new.env(parent = emptyenv())
     action_name <- unique_action_name()
