@@ -24,41 +24,41 @@ ok(cmp_environment(stock_a$env, list(
     stock__area = 1L,
     stock__area_idx = stock_a$env$stock__area_idx,
     stock__totalareas = 1L,
-    stock__areas = as.array(c(a = 1L)),
+    stock__areas = gadget3:::force_vector(a = 1L),
     area_a = 1L,
     stock__upperlen = Inf,
-    stock__minlen = as.array(c("10:Inf" = 10)),
-    stock__midlen = as.array(c("10:Inf" = 10.5)),
+    stock__minlen = gadget3:::force_vector("10:Inf" = 10),
+    stock__midlen = gadget3:::force_vector("10:Inf" = 10.5),
     stock__maxmidlen = 10.5,
     stock__minmidlen = 10.5,
-    stock__maxlen = as.array(c("10:Inf" = Inf)),
+    stock__maxlen = gadget3:::force_vector("10:Inf" = Inf),
     stock__plusdl = 1,
     stock__dl = 1)), "stock_a: Environment populated with relevant areas")
 ok(cmp_environment(stock_ac$env, list(
     stock__totalareas = 2L,
-    stock__areas = as.array(c(area1 = 1L, area3 = 3L)),
+    stock__areas = gadget3:::force_vector(area1 = 1L, area3 = 3L),
     area_area1 = 1L,
     area_area3 = 3L,
     stock__upperlen = Inf,
-    stock__minlen = as.array(c("10:Inf" = 10)),
-    stock__midlen = as.array(c("10:Inf" = 10.5)),
+    stock__minlen = gadget3:::force_vector("10:Inf" = 10),
+    stock__midlen = gadget3:::force_vector("10:Inf" = 10.5),
     stock__maxmidlen = 10.5,
     stock__minmidlen = 10.5,
-    stock__maxlen = as.array(c("10:Inf" = Inf)),
+    stock__maxlen = gadget3:::force_vector("10:Inf" = Inf),
     stock__plusdl = 1,
     stock__dl = 1)), "stock_c: Environment populated with default areas")
 ok(cmp_environment(stock_bcd$env, list(
     stock__totalareas = 3L,
-    stock__areas = as.array(c(b = 2L, c = 3L, d = 4L)),
+    stock__areas = gadget3:::force_vector(b = 2L, c = 3L, d = 4L),
     area_b = 2L,
     area_c = 3L,
     area_d = 4L,
     stock__upperlen = Inf,
-    stock__minlen = as.array(c("10:Inf" = 10)),
-    stock__midlen = as.array(c("10:Inf" = 10.5)),
+    stock__minlen = gadget3:::force_vector("10:Inf" = 10),
+    stock__midlen = gadget3:::force_vector("10:Inf" = 10.5),
     stock__maxmidlen = 10.5,
     stock__minmidlen = 10.5,
-    stock__maxlen = as.array(c("10:Inf" = Inf)),
+    stock__maxlen = gadget3:::force_vector("10:Inf" = Inf),
     stock__plusdl = 1,
     stock__dl = 1)), "stock_a: Environment populated with relevant areas")
     
